"use strict";
/* eslint-disable @typescript-eslint/ban-types */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = __importDefault(require("debug"));
const fs_1 = __importDefault(require("fs"));
const git_config_1 = __importDefault(require("git-config"));
const lodash_1 = __importDefault(require("lodash"));
const moment_1 = __importDefault(require("moment"));
const path_1 = __importDefault(require("path"));
const swig_templates_1 = __importDefault(require("swig-templates"));
const yeoman_generator_1 = __importDefault(require("yeoman-generator"));
const debug = (0, debug_1.default)('yo:magicdawn:app');
class AppGeneratorLogic extends yeoman_generator_1.default {
    constructor(args, opts) {
        super(args, opts);
        // eslint-disable-next-line prefer-rest-params
        debug('constructor arguments %j', arguments);
        this.sourceRoot(path_1.default.join(__dirname, '../../templates/app/'));
    }
    default() {
        const ok = this.checkPackageJson();
        if (!ok)
            return;
        // 生成package.json
        //    deps
        //    scripts test / test-cover
        this.modifyPackageJson();
        // 复制文件
        this.copyFiles();
    }
    /**
     * 检查 `package.json` 文件
     */
    checkPackageJson() {
        const destPackageJsonPath = this.destinationPath('package.json');
        const exists = fs_1.default.existsSync(destPackageJsonPath);
        // warn
        if (!exists) {
            console.error('\n[error]: package.json not found, run `npm init` first');
            return false;
        }
        // ok
        return true;
    }
    /**
     * 修改 package.json
     *
     * 	- deps
     * 	- scripts.{ test, test-cover }
     */
    modifyPackageJson() {
        const pkg = this.fs.readJSON(this.templatePath('package.json'));
        const pkgPartial = lodash_1.default.omit(pkg, ['name', 'version', 'private']);
        this.fs.extendJSON(this.destinationPath('package.json'), pkgPartial);
    }
    /**
     * 复制文件
     */
    copyFiles() {
        const locals = this.getTemplateLocals();
        const files = [
            // eslint
            '.eslintrc.yml',
            '.eslintignore',
            // edit
            '.editorconfig',
            // prettier
            '.prettierignore',
            'prettier.config.js',
            '.husky',
            // test
            'test/.gitkeep',
            // ci
            '.github',
            // package related
            { name: 'LICENSE' },
            { name: 'CHANGELOG.md' },
            { name: 'README.md' },
            // ts
            'src',
            'tsconfig.json',
        ];
        for (const f of files) {
            // write as same
            if (typeof f === 'string') {
                const from = this.templatePath(f);
                const to = this.destinationPath(f);
                this.fs.copy(from, to);
            }
            // render & write
            else {
                const { name, locals: currentLocals } = f;
                const from = this.templatePath(name);
                const to = this.destinationPath(name);
                const content = swig_templates_1.default.renderFile(from, currentLocals ?? locals);
                this.fs.write(to, content);
            }
        }
        // .gitignore 特殊
        // https://github.com/npm/npm/issues/3763
        this.fs.copy(this.templatePath('gitignore'), this.destinationPath('.gitignore'));
    }
    getTemplateLocals() {
        const pkg = this.fs.readJSON(this.destinationPath('package.json'));
        // 获取 repoName
        const getRepoName = () => {
            let repoName;
            const giturl = git_config_1.default.sync(process.cwd() + '/.git/config')?.['remote "origin"']?.url;
            repoName = giturl && path_1.default.basename(giturl, '.git');
            if (repoName) {
                debug('repoName from git url: %s', repoName);
                return repoName;
            }
            // pkg.name as fallback
            repoName = pkg.name;
            debug('repoName from pkg.name: %s', repoName);
            return repoName;
        };
        const currentYear = (0, moment_1.default)().format('YYYY');
        const currentDate = (0, moment_1.default)().format('YYYY-MM-DD');
        const packageName = pkg.name;
        const packageLocalName = lodash_1.default.camelCase(pkg.name); // 变量名
        const packageDescription = pkg.description || ''; // 描述
        return {
            currentYear,
            currentDate,
            packageName,
            repoName: getRepoName(),
            packageLocalName,
            packageDescription,
        };
    }
}
class AppGenerator extends AppGeneratorLogic {
    default() {
        return super.default();
    }
}
exports.default = AppGenerator;
